@@ -75,7 +75,7 @@
 #'
 #' @examples
 #'
-#' # Create fast-and-frugal trees (FFTs) for heart disease:
+#' # 1. Create fast-and-frugal trees (FFTs) for heart disease:
 #' heart.fft <- FFTrees(formula = diagnosis ~ .,
 #'                      data = heart.train,
 #'                      data.test = heart.test,
@@ -83,22 +83,23 @@
 #'                      decision.labels = c("Healthy", "Diseased")
 #'                      )
 #'
-#' # Print a summary of the result:
-#' heart.fft
+#' # 2. Print a summary of the result:
+#' heart.fft  # same as:
+#' # print(heart.fft, data = "train", tree = "best.train")
 #'
-#' # Plot an FFT applied to training data:
-#' plot(heart.fft, stats = FALSE)
-#' plot(heart.fft)
+#' # 3. Plot an FFT applied to training data:
+#' plot(heart.fft)  # same as:
+#' # plot(heart.fft, what = "all", data = "train", tree = "best.train")
 #'
-#' # Apply FFT to (new) testing data:
-#' plot(heart.fft, data = "test")
-#' plot(heart.fft, data = "test", tree = 2)  # Plot Tree #2
+#' # 4. Apply FFT to (new) testing data:
+#' plot(heart.fft, data = "test")            # predictions for Tree 1
+#' plot(heart.fft, data = "test", tree = 2)  # predictions for Tree 2
 #'
-#' # Predict classes and probabilities for new data:
+#' # 5. Predict classes and probabilities for new data:
 #' predict(heart.fft, newdata = heartdisease)
 #' predict(heart.fft, newdata = heartdisease, type = "prob")
 #'
-#' # Create custom trees with my.tree:
+#' # 6. Create custom trees (from verbal description) with my.tree:
 #' custom.fft <- FFTrees(
 #'   formula = diagnosis ~ .,
 #'   data = heartdisease,
@@ -277,6 +278,7 @@ FFTrees <- function(formula = NULL,
 
   x <- fftrees_ffttowords(
     x = x,
+    mydata = "train",  # either 'train':'decide' or 'test':'predict'
     digits = 2
   )
 
